/******************************************************************************\
 * Technische Universitaet Darmstadt, Institut fuer Nachrichtentechnik
 * Copyright (c) 2001
 *
 * Author(s):
 *	Volker Fischer
 *
 * Description:
 *	See AMDemodulation.cpp
 *
 ******************************************************************************
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later 
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
\******************************************************************************/

#if !defined(AMDEMOD_H__3B0BEVJBN8LKH2934BGF4344_BB27912__INCLUDED_)
#define AMDEMOD_H__3B0BEVJBN8LKH2934BGF4344_BB27912__INCLUDED_

#include "Parameter.h"
#include "Modul.h"
#include "Vector.h"
#include "matlib/Matlib.h"
#include "sync/TimeSyncFilter.h"

#ifdef HAVE_DRFFTW_H
# include <drfftw.h>
#else
# include <rfftw.h>
#endif


/* Definitions ****************************************************************/
/* Set the number of blocks used for carrier frequency acquisition */
#define NUM_BLOCKS_CARR_ACQUISITION			10


/* Classes ********************************************************************/
class CAMDemodulation : public CReceiverModul<_REAL, _SAMPLE>
{
public:
	CAMDemodulation() : bAcquisition(TRUE) {}
	virtual ~CAMDemodulation() {}

	void SetFilterTaps(_REAL rNewOffsetNorm);

protected:
	CRealVector					rvecA;
	CRealVector					rvecBReal;
	CRealVector					rvecBImag;
	CRealVector					rvecZReal;
	CRealVector					rvecZImag;
	CRealVector					rvecInpTmp;
	CComplexVector				cvecHilbert;

	CComplex					cCurExp;
	CComplex					cExpStep;

	int							iSymbolBlockSize;

	_BOOLEAN					bAcquisition;
	CShiftRegister<fftw_real>	vecrFFTHistory;

	CFftPlans					FftPlan;
	CRealVector					vecrFFTInput;
	CComplexVector				veccFFTOutput;

	int							iTotalBufferSize;
	int							iAquisitionCounter;
	CRealVector					vecrPSD;
	int							iHalfBuffer;


	virtual void InitInternal(CParameter& ReceiverParam);
	virtual void ProcessDataInternal(CParameter& ReceiverParam);
};


#endif // !defined(AMDEMOD_H__3B0BEVJBN8LKH2934BGF4344_BB27912__INCLUDED_)
